# *************************************************************************
# ATTENTION: This is a BETA version of LAM/MPI!  It is intended for
# early adopters and testers; it is not intended for production runs.
# This distribution is *not* guaranteed to be stable.  If you are
# looking for a robust version of MPI for day-to-day use, you should
# download the latest stable version from the main LAM/MPI web site:
# 			http://www.lam-mpi.org/
# *************************************************************************
# 
# Copyright 2001-2002, Indiana University
# 
# Authors: Jeffrey M. Squyres and Brian Barrett with Andrew Lumsdaine,
#          and contributions from M. D. McNally, Kinis L. Meyer, Arun
#          Rodrigues, and Jeremy G. Siek.
# 
# This file is part of the Indiana University LAM implementation of MPI.
# 
# You should have received a copy of the License Agreement for the
# Indiana University LAM implementation of MPI along with the software;
# see the file LICENSE.  If not, see LAM/MPI's main web site:
# http://www.lam-mpi.org/.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted subject to the conditions specified in the
# LICENSE file.
# 
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# 
# Additional copyrights may follow.
# 
#
#	Ohio Trollius
#
#	Function:	- bhost.def
#			- a single-node LAM
#

localhost
