// -*- c++ -*-
//
// $Id: functions.h,v 1.4 2001/12/15 02:42:42 jsquyres Exp $
//
// $COPYRIGHT$
//

//
// Point-to-Point Communication
//

_MPIPP_STATIC_ void 
Attach_buffer(void* buffer, int size);

_MPIPP_STATIC_ int 
Detach_buffer(void*& buffer);

//
// Process Topologies
//

_MPIPP_STATIC_ void
Compute_dims(int nnodes, int ndims, int dims[]);

//
// Environmental Inquiry
//

_MPIPP_STATIC_ void 
Get_processor_name(char* name, int& resultlen);

_MPIPP_STATIC_ void
Get_error_string(int errorcode, char* string, int& resultlen);

_MPIPP_STATIC_ int 
Get_error_class(int errorcode);

_MPIPP_STATIC_ double 
Wtime();

_MPIPP_STATIC_ double 
Wtick();

_MPIPP_STATIC_ void
Init(int& argc, char**& argv);

_MPIPP_STATIC_ void
Init();

_MPIPP_STATIC_ void
Real_init();

_MPIPP_STATIC_ void
Finalize();

_MPIPP_STATIC_ MPI2CPP_BOOL_T
Is_initialized();

//
// Profiling
//

_MPIPP_STATIC_ void
Pcontrol(const int level, ...);


#if MPI2CPP_HAVE_MPI_GET_VERSION
_MPIPP_STATIC_ void
Get_version(int& version, int& subversion);
#endif

_MPIPP_STATIC_ _REAL_MPI_::Aint
Get_address(void* location);
